/*
 * Copyright (c) 1997 University of Southern California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Information Sciences
 *      Institute of the University of Southern California.
 * 4. Neither the name of the University nor of the Institute may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include "lan.h"
#include "edge.h"
#include "node.h"

LanLink::LanLink(Edge *e): edge_(e)
{
}

Lan::Lan(const char *name, double ps, double bw, double delay, double angle) :
  Animation(0,0), links_(NULL), ps_(ps), bw_(bw), delay_(delay), angle_(angle),
  marked_(0)
{
  virtual_node_=new VirtualNode(name);
  name_ = new char[strlen(name) + 1];
  strcpy(name_, name);
  ln_ = atoi(name); /*XXX*/

}

Lan::~Lan() {
}

void Lan::add_node(NetModel */*nm*/, Node *n, double angle) {
  printf("lan::add_node %s\n", n->getname());
  no_of_nodes_++;
  Edge *e1=new Edge(n, virtual_node_, ps_, bw_, delay_/2.0, angle);
  Edge *e2=new Edge(virtual_node_, n, ps_, bw_, delay_/2.0, angle);
  /*
  nm->enterEdge(e1);
  nm->enterEdge(e2);
  */
  LanNode *ln = new LanNode(n, angle, e1, e2);
  ln->next_node(nodes_);
  nodes_=ln;
  n->add_link(e1);
  virtual_node_->add_link(e2);
}

void Lan::add_link(Edge *e) {
  LanLink *ll = new LanLink(e);
  ll->next(links_);
  links_=ll;
  virtual_node_->add_link(e);
}


void Lan::draw(class NetView */*nv*/, double /*time*/) const {
  printf("drawing LAN\n");
}

Edge *Lan::lookupEdge(Node *n) {
  LanNode *ln=nodes_;
  while(ln!=NULL) {
    if (ln->node()==n)
    {
      return ln->e1_;
    }
  }
  return NULL;
}
